<?php

if(!class_exists('lumise_connector')){

	class lumise_connector {
        
        protected $woo;
        
        public $platform;
        
        public function __construct() {

            global $lumise, $lumise_woo, $wpdb;

            $this->woo = $lumise_woo;
            $this->platform = 'woocommerce';
            
            $admin_url = admin_url('admin.php?page=lumise&');
            
            $this->config = array(
				"url" => $lumise_woo->tool_url,
				"tool_url" => $lumise_woo->tool_url,
				"logo" => $lumise_woo->assets_url . 'assets/images/logo.v3.png',
				"ajax_url" => $lumise_woo->ajax_url,
				"admin_ajax_url" => $lumise_woo->admin_ajax_url,
				"assets_url" => $lumise_woo->assets_url,
				"load_jquery" => true,
				"root_path" => $lumise_woo->app_path,
				"upload_path" => $lumise_woo->upload_path,
				"upload_url" => $lumise_woo->upload_url,
				"checkout_url" => $lumise_woo->checkout_url,
                
                //admin setting
                "admin_assets_url" => $lumise_woo->admin_assets_url,
                "admin_url" => $admin_url,
				
				"database" => array(
                    "host" => DB_HOST,
                    "user" => DB_USER,
                    "pass" => DB_PASSWORD,
                    "name" => DB_NAME,
					"prefix" => 'lumise_'
                )
			);
			
			$lumise->add_filter('product_price', array(&$this, 'filter_product_price'));
			$lumise->add_filter('product', array(&$this, 'filter_product'));
			$lumise->add_filter('products', array(&$this, 'filter_products'));
			$lumise->add_filter('order_status', array(&$this, 'order_status_name'));
			$lumise->add_action('before_order_products', array(&$this, 'update_order_status'));
			$lumise->add_filter('product_base_price', array(&$this, 'product_base_price'));
			$lumise->add_filter('product_templates', array(&$this, 'product_templates'));
			$lumise->add_filter('setting_tabs', array(&$this, 'setting_tabs'));
			$lumise->add_filter('product_tabs', array(&$this, 'product_tabs'));
			$lumise->add_filter('init_settings', array(&$this, 'init_settings'));
			$lumise->add_filter('after_save_settings', array(&$this, 'after_save_settings'));
            
        }

        public function get_session($name) {
            return isset($_SESSION[$name]) ? $_SESSION[$name] : null;
        }

        public function set_session($name, $value) {
            $_SESSION[$name] = $value;
        }

        public function cookie($name) {
            return isset($_COOKIE[$name]) ? $_COOKIE[$name] : null;
        }
		
        public function is_admin() {
			
			return current_user_can('administrator')? true : false;
		}
		
		public function is_login() {
			global $lumise;
			return $lumise->connector->cookie('uid') || 0;
			
		}
		
        public function capabilities($cap) {
		    
		    return current_user_can($cap);
		    
	    }
	    
		public function filter_product_price($price){
			if(function_exists('wc_price'))
				return wc_price( $price);
			else
				return $price;
		}
        
        public function filter_product($data) {
			
			/*
			*	$data['product'] is product CMS ID
			*	if not isset --> the product base can not checkout (missing cms product)
			*/
			if (!isset($data['product']) || empty($data['product']))
				return null;
			
			/*
			*	Incase force a product base with any product CMS 
			*/	
			if(isset($_GET['product_cms']) && $_GET['product_cms'] != $data['product'])
				$data['product'] = $_GET['product_cms'];
			
			
			global $wpdb, $lumise;
			
			$cms_product = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}posts` WHERE `id`=".$data['product']);
			$cms_template = get_post_meta($data['product'], 'lumise_design_template', true );
			
			if (!isset($cms_product[0]))
				return null;
			
			if(function_exists('wc_get_product')){
				$_product = wc_get_product( $data['product'] );
				if(!empty($_product)) $data['price'] = $_product->get_price();
			}
				
			$data['templates'] = array();
			if (isset($cms_template) && !empty($cms_template) && $cms_template != '%7B%7D') {
				$cms_template = json_decode(urldecode($cms_template), true);
				$stages = $lumise->lib->dejson($data['stages']);
				foreach ($stages as $s => $d) {
					if (isset($cms_template[$s])){
						$stages->{$s}->template = $cms_template[$s];
					}
					else unset($stages->{$s}->template);
				}
				$data['templates'] = $cms_template;
				$data['stages'] = $lumise->lib->enjson($stages);
			}
			$data['name'] = $lumise->lang($cms_product[0]->post_title);
			$data['sku'] = get_post_meta($data['product'], '_sku', true);;
			$data['description'] = $lumise->lang($cms_product[0]->post_content);
			
			
			
	        return $data;
	        
        }
                
        public function filter_products($products){
			
			
			$results = array();
			global $wpdb, $lumise;
			
			$task = $lumise->lib->esc('task');
			
			/*
			*	Select all product base	for task on CMS product
			*/
			
			if ($task == 'cms_product')
				return $products;
						
			if (count($products) > 0) {
				
				foreach ($products as $key => $data) {
					
					if (isset($data['product']) && !empty($data['product'])) {
						
						$cms_product = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}posts` WHERE `post_status` = 'publish' AND `id`=".$data['product']);
						
						if (isset($cms_product[0])) {
							$data['name'] = $lumise->lang($cms_product[0]->post_title);
							$data['description'] = $lumise->lang($cms_product[0]->post_content);
							if(function_exists('wc_get_product')){
								$_product = wc_get_product( $data['product'] );
								if(!empty($_product)) $data['price'] = $_product->get_price();
							}
							
							//get list of templates
							$cms_template = get_post_meta($data['product'], 'lumise_design_template', true );
							$data['templates'] = array();
							
							if (isset($cms_template) && !empty($cms_template) && $cms_template != '%7B%7D') {
								$cms_template = json_decode(urldecode($cms_template), true);
								$data['templates'] = $cms_template;
							}
							
							array_push($results, $data);
							
						}
						
					}
				}
			}
			
	        return $results;
	        
        }
        
        public function orders($filter, $orderby, $ordering, $limit, $limit_start) {

            global $wpdb;

            $items = array('rows' => array());

            $ops = $this->config['database']['prefix'] . 'order_products';
            $wois = $wpdb->prefix . 'woocommerce_order_items';
            $woim = $wpdb->prefix . 'woocommerce_order_itemmeta';
            $posts = $wpdb->prefix . 'posts';
            $pom = $wpdb->prefix . 'postmeta';
			
			$items['rows'] = array();
			$total_count = 0;
			
			if(class_exists( 'WooCommerce' )){
				$where = array();
				$where[] = " pos.post_status <> 'trash'";
				
	            if (is_array($filter) && isset($filter['keyword']) && !empty($filter['keyword'])) {

	                $fields = explode(',', $filter['fields']);
	                $arr_keyword = array();
	                for ($i = 0; $i < count($fields); $i++) {
	                    $arr_keyword[] = sprintf(" %s LIKE '%s' ", $fields[$i], $filter['keyword']);
	                }

	                $fields = implode(' OR ', $arr_keyword);

	                $where[] = $fields;
	            }
				
				if (count($where) > 0)
					$where = (count($where) > 0) ? ' WHERE ' . implode(' AND ', $where) : '';

	            $orderby_str = '';
	            if ($orderby != null && $ordering != null)
	                $orderby_str = ' ORDER BY ' . $orderby . ' ' . $ordering;

	            $sql = "SELECT SQL_CALC_FOUND_ROWS " 
	                . " wois.*, pos.ID as id, wois.order_item_name as product_name, pos.post_status as status, pos.post_date as created, pos.post_modified as updated, max( CASE WHEN pom.meta_key = '_order_total' and pos.ID = pom.post_id THEN pom.meta_value END ) as total, max( CASE WHEN pom.meta_key = '_order_currency' and pos.ID = pom.post_id THEN pom.meta_value END ) as currency"
	                . " FROM $ops as ops "
	                . " LEFT JOIN $wois as wois ON ops.order_id = wois.order_id"
	                . " LEFT JOIN $posts as pos ON pos.ID = ops.order_id"
	                . " LEFT JOIN $pom as pom on pos.ID = pom.post_id"
	                . $where
	                . " GROUP BY pos.ID "
	                . $orderby_str
	                . " LIMIT $limit_start, $limit";
					
	            $rows = $wpdb->get_results($sql);
				
	            foreach ($rows as $row)
	                $items['rows'][] = (array) $row;

	            //get total 
	            
	            $sql = "SELECT FOUND_ROWS() ";

	            $total_count = $wpdb->get_var($sql);
			}
            

            $items['total_count'] = $total_count;
            $items['total_page'] = ($limit != null) ? ceil($total_count / $limit) : 0;
            return $items;
        }

        public function products_order($order_id, $filter, $orderby, $ordering) {
            global $wpdb;

            $items = array('rows' => array());

            $ops = $this->config['database']['prefix'] . 'order_products';
            $wois = $wpdb->prefix . 'woocommerce_order_items';
            
            $where = array();
            
            $where[] = 'ops.order_id = '. $order_id;

            if (is_array($filter) && isset($filter['keyword']) && !empty($filter['keyword'])) {

                $where = array();
                $fields = explode(',', $filter['fields']);
                $arr_keyword = array();
                for ($i = 0; $i < count($fields); $i++) {
                    $arr_keyword[] = sprintf(" %s LIKE '%s' ", $fields[$i], $filter['keyword']);
                }

                $fields = '(' . implode(' OR ', $arr_keyword) . ')';

                $where[] = $fields;
                    
            }
            

            $orderby_str = '';
            if ($orderby != null && $ordering != null)
                $orderby_str = ' ORDER BY ' . $orderby . ' ' . $ordering;

            $sql = "SELECT "
                . "SQL_CALC_FOUND_ROWS *"
                . " FROM $ops as ops "
                . " JOIN $wois as wois ON wois.order_id = ops.order_id"
                . ' WHERE '. implode(' AND ', $where)
                . " GROUP BY ops.id "
                . $orderby_str;

            $rows = $wpdb->get_results($sql);

            foreach ($rows as $row)
                $items['rows'][] = (array) $row;

            //get total 
            $sql = "SELECT FOUND_ROWS() ";

            $total_count = $wpdb->get_var($sql);

            $items['total_count'] = $total_count;
            $items['total_page'] = 1;
			
			$order = new WC_Order( $order_id );
			$created_date = date('F j, Y, g:i a', strtotime($order->get_date_created()));
			$modified_date = date('F j, Y, g:i a', strtotime($order->get_date_modified()));
			
			$items['order'] = array(
				'created' => $created_date,
				'updated' => $modified_date,
				'total' => $order->get_total(),
				'status' => 'wc-' . $order->get_status()
			);
			
            return $items;
        }

		public function add_to_cart($data){
			
			global $woocommerce, $lumise, $lumise_cart_adding;
			
			$lumise_cart_adding = true;
			
			
			if(is_array($data) && isset($woocommerce)){
				//remove all items from editor
				foreach( WC()->cart->get_cart() as $cart_item_key => $item ){
					if( 
						isset($item['lumise_data'])
						&& (
							(isset( $item['lumise_incart'] ) && !$item['lumise_incart'] )
							|| !isset( $item['lumise_incart'] ) )
					) $woocommerce->cart->remove_cart_item( $cart_item_key );
				}
				
				$removed_items = ($lumise->connector->get_session('lumise_cart_removed') == null)? array() : $lumise->connector->get_session('lumise_cart_removed');
				
				//update items
				$cart_ids = array();
				
				foreach($data as $cart_id => $item){
					
					//get item data from file
					
					if( !isset( $item[ 'incart' ] ) ){
						$product_id = $item[ 'product_cms' ];
						
						$extra_option = array(
							'lumise_data' => array(
								'cart_id' => $cart_id,
								'file'	=> $item[ 'file' ]
							) 
						);
						
						$woocommerce->cart->add_to_cart( $product_id, 1 , null, null, $extra_option );

					}
					$cart_ids[] = $item['cart_id'];
				}
				
				$removed_items = array_diff( $removed_items, $cart_ids );
				
				$lumise->connector->set_session('lumise_cart_removed', $removed_items );
				
			}

			$cart_url = wc_get_cart_url();
			if ( wp_redirect( $cart_url ) ) {
			    exit;
			}
			return true;
		}
		
		public function statuses(){
			return wc_get_order_statuses();
		}
		
		public function update_order_status($order_id){
			if(isset($_POST['order_status'])){
				$status = str_replace('wc-', '', $_POST['order_status']);
				$order = new WC_Order($order_id);
				$order->update_status($status);
			}
		}
		
		public function order_status_name($status){
			return (function_exists('wc_get_order_status_name'))? wc_get_order_status_name($status) : $status;
		}
		
		public function product_base_price($data){
			global $lumise;
			
			$product_id = $data[1];
			$price = $data[0];
			if(function_exists('wc_get_product')){
				$_product = wc_get_product( $product_id );
				$price = ($_product)? $_product->get_price(): 0;
			}
			return array($price, $product_id);
		}
		
		public function product_templates($data){
			$item = $data['item'];
			
			if($item->cms_id){
				
				$cms_template = get_post_meta($item->cms_id, 'lumise_design_template', true );
				
				$template_price = 0;
				if (isset($cms_template) && !empty($cms_template) && $cms_template != '%7B%7D') {
					$data['templates'] = array();
					$cms_template = json_decode(urldecode($cms_template), true);
					
					foreach($cms_template as $stage){
						if(!in_array($stage['id'], $data['templates'])){
							$data['templates'][] = $stage['id'];
						}
					}
					
				}
			}
			
			return $data['templates'];
		}
		
		public function product_tabs($arg)
		{
			global $lumise, $wpdb;
			if(isset($_GET['id'])){
				$product = $_GET['id'];

				foreach($arg['tabs']['details:' . $lumise->lang('Details')] as $index => $field){
					if(
					 	$field['name'] == 'product' &&
						$product > 0
					){
						//find all product assigned
						$args = array (
						    'post_type'		=> array( 'product' ),
						    'meta_query'	=> array(
						        array(
						            'key'       => 'lumise_product_base',
						            'value'     => $product,
						        ),
						    ),
						);
						$query = new WP_Query( $args );
						$products = array();
						if ( $query->have_posts() ) {
						    while ( $query->have_posts() ) {
						        $query->the_post();
						        $products[get_the_ID()] = get_the_title();
						    }
							
							$arg['tabs']['details:' . $lumise->lang('Details')][$index]['options'] = $products;
							$arg['tabs']['details:' . $lumise->lang('Details')][$index]['type'] = 'dropbox';
						}
						
						
					}
				}
			}
			return $arg;
		}
		
		public function setting_tabs($arg){
			global $lumise;
			
			$pages = get_pages();
			
			$pages_list = array();
			
			foreach ($pages as $page) {
				$pages_list[$page->ID] = $page->post_title;
			}
			
			$arg['tabs']['shop:'.$lumise->lang('Shop')][] = array(
				'type' 		=> 'dropbox',
				'name' 		=> 'editor_page',
				'value' 	=> '',
				'label' 	=> $lumise->lang('Editor Page'),
				'desc' 		=> $lumise->lang('Page to display Lumise editor for design.'),
				'options' 	=> $pages_list
			);
			
			$arg['tabs']['shop:'.$lumise->lang('Shop')][] = array(
				'type' => 'input',
				'name' => 'btn_text',
				'value' => 'Customize',
				'label' => $lumise->lang('Button Text'),
				'desc' => $lumise->lang('Customize button text. Default: Customize'),
			);
			
			$arg['tabs']['shop:'.$lumise->lang('Shop')][] = array(
				'type' => 'toggle',
				'name' => 'btn_list',
				'value' => 1,
				'label' => $lumise->lang('Button Listing'),
				'desc' => $lumise->lang('Show Customize button on list products page or other listing'),
			);
			
			$arg['tabs']['shop:'.$lumise->lang('Shop')][] = array(
				'type' => 'toggle',
				'name' => 'btn_page',
				'value' => 1,
				'label' => $lumise->lang('Button Product Page'),
				'desc' => $lumise->lang('Show Customize button on product page'),
			);
			
			$arg['tabs']['shop:'.$lumise->lang('Shop')][] = array(
				'type' => 'toggle',
				'name' => 'email_design',
				'value' => 1,
				'label' => $lumise->lang('Email Customer Link'),
				'desc' => $lumise->lang('Allow customers view there design after order completed.'),
			);
			
			return $arg;
		}
		
		public function init_settings($settings){
			
			$settings = array_merge($settings, array(
				'editor_page' => get_option('lumise_editor_page', 0),
				'btn_list' => 1,
				'btn_text' => 'Customize',
				'btn_page' => 1,
				'email_design' => 1
			));
			
			return $settings;
		}
		
		public function after_save_settings($data){
			//update page editor
			$fields = array(
				'btn_list',
				'btn_text',
				'btn_page',
				'email_design',
			);
			
			$config = array();
			foreach($fields as $field){
				$config[$field] = $data[$field];
			}
			update_option('lumise_editor_page', $data['editor_page']);
			update_option('lumise_config', $config);
		}
		
		public function update() {
			
			global $lumise;
			
			$lumise_path = dirname(__FILE__);
			$update_path = $lumise->cfg->upload_path.'tmpl'.DS.'lumise';
			$backup_path = $lumise->cfg->upload_path.'update_backup';
			
			$lumise->lib->delete_dir($backup_path);
								
			return (
				@rename($lumise_path, $backup_path) && 
				@rename($update_path, $lumise_path)
			);
			
		}

    }

}
